@@ -34,6 +34,8 @@
 #include <libtracker-fts/tracker-fts.h>
 #endif
 
+#include "tracker-collation.h"
+
 #include "tracker-db-interface-sqlite.h"
 
 struct TrackerDBInterface {
@@ -46,11 +48,17 @@ struct TrackerDBInterface {
 
 	GSList *function_data;
 
+	gpointer collator;
+
 	guint ro : 1;
 #if HAVE_TRACKER_FTS
-	guint fts_initialized : 1;
+	TrackerFts *fts;
 #endif
 	GCancellable *cancellable;
+
+	/* async operation pending */
+	gboolean pending;
+	GAsyncReadyCallback outstanding_callback;
 };
 
 struct TrackerDBInterfaceClass {
@@ -62,6 +70,8 @@ struct TrackerDBCursor {
 	sqlite3_stmt *stmt;
 	TrackerDBStatement *ref_stmt;
 	gboolean finished;
+	TrackerPropertyType *types;
+	gint n_types;
 };
 
 struct TrackerDBCursorClass {
@@ -79,11 +89,18 @@ struct TrackerDBStatementClass {
 	GObjectClass parent_class;
 };
 
-static TrackerDBStatement       * tracker_db_statement_sqlite_new (TrackerDBInterface     *db_interface,
-                                                                   sqlite3_stmt           *sqlite_stmt);
-static TrackerDBCursor          * tracker_db_cursor_sqlite_new    (sqlite3_stmt           *sqlite_stmt,
-                                                                   TrackerDBStatement     *ref_stmt);
-static void tracker_db_statement_sqlite_reset (TrackerDBStatement *stmt);
+static TrackerDBStatement *tracker_db_statement_sqlite_new   (TrackerDBInterface   *db_interface,
+                                                              sqlite3_stmt         *sqlite_stmt);
+static void                tracker_db_statement_sqlite_reset (TrackerDBStatement   *stmt);
+static TrackerDBCursor    *tracker_db_cursor_sqlite_new      (sqlite3_stmt         *sqlite_stmt,
+                                                              TrackerDBStatement   *ref_stmt,
+                                                              TrackerPropertyType  *types,
+                                                              gint                  n_types);
+static gboolean            tracker_db_cursor_get_boolean     (TrackerSparqlCursor  *cursor,
+                                                              guint                 column);
+static gboolean            db_cursor_iter_next               (TrackerDBCursor      *cursor,
+                                                              GCancellable         *cancellable,
+                                                              GError              **error);
 
 enum {
 	PROP_0,
@@ -499,6 +516,20 @@ open_database (TrackerDBInterface *db_interface)
 		g_message ("Opened sqlite3 database:'%s'", db_interface->filename);
 	}
 
+	/* Set our unicode collation function */
+	if (!db_interface->collator) {
+		db_interface->collator = tracker_collation_init ();
+		if (sqlite3_create_collation (db_interface->db,
+		                              TRACKER_COLLATION_NAME,
+		                              SQLITE_UTF8,
+		                              db_interface->collator,
+		                              tracker_collation_utf8) != SQLITE_OK)
+		{
+			g_critical ("Couldn't set collation function: %s",
+			            sqlite3_errmsg (db_interface->db));
+		}
+	}
+
 	sqlite3_progress_handler (db_interface->db, 100,
 	                          check_interrupt, db_interface);
 
@@ -609,8 +640,8 @@ close_database (TrackerDBInterface *db_interface)
 	db_interface->function_data = NULL;
 
 #if HAVE_TRACKER_FTS
-	if (db_interface->fts_initialized) {
-		tracker_fts_shutdown (G_OBJECT (db_interface));
+	if (db_interface->fts) {
+		tracker_fts_free (db_interface->fts);
 	}
 #endif
 
@@ -623,13 +654,43 @@ tracker_db_interface_sqlite_fts_init (TrackerDBInterface *db_interface,
                                       gboolean            create)
 {
 #if HAVE_TRACKER_FTS
-	tracker_fts_init (db_interface->db, create, G_OBJECT (db_interface));
-	db_interface->fts_initialized = TRUE;
+	db_interface->fts = tracker_fts_new (db_interface->db, create);
 #else
 	g_message ("FTS support is disabled");
 #endif
 }
 
+#if HAVE_TRACKER_FTS
+int
+tracker_db_interface_sqlite_fts_update_init (TrackerDBInterface *db_interface,
+                                             int                 id)
+{
+	return tracker_fts_update_init (db_interface->fts, id);
+}
+
+int
+tracker_db_interface_sqlite_fts_update_text (TrackerDBInterface *db_interface,
+                                             int                 id,
+                                             int                 column_id,
+                                             const char         *text,
+                                             gboolean            limit_word_length)
+{
+	return tracker_fts_update_text (db_interface->fts, id, column_id, text, limit_word_length);
+}
+
+void
+tracker_db_interface_sqlite_fts_update_commit (TrackerDBInterface *db_interface)
+{
+	return tracker_fts_update_commit (db_interface->fts);
+}
+
+void
+tracker_db_interface_sqlite_fts_update_rollback (TrackerDBInterface *db_interface)
+{
+	return tracker_fts_update_rollback (db_interface->fts);
+}
+#endif
+
 static void
 tracker_db_interface_sqlite_finalize (GObject *object)
 {
@@ -643,6 +704,8 @@ tracker_db_interface_sqlite_finalize (GObject *object)
 
 	g_free (db_interface->filename);
 
+	tracker_collation_shutdown (db_interface->collator);
+
 	G_OBJECT_CLASS (tracker_db_interface_parent_class)->finalize (object);
 }
 
@@ -760,6 +823,7 @@ tracker_db_interface_create_statement (TrackerDBInterface  *db_interface,
 	gchar *full_query;
 
 	g_return_val_if_fail (TRACKER_IS_DB_INTERFACE (db_interface), NULL);
+	g_return_val_if_fail (!db_interface->pending, NULL);
 
 	va_start (args, query);
 	full_query = g_strdup_vprintf (query, args);
@@ -974,34 +1038,6 @@ tracker_db_interface_sqlite_new_ro (const gchar *filename)
 	                     NULL);
 }
 
-static gint
-collation_function (gpointer      data,
-                    int           len1,
-                    gconstpointer str1,
-                    int           len2,
-                    gconstpointer str2)
-{
-	TrackerDBCollationFunc func;
-
-	func = (TrackerDBCollationFunc) data;
-
-	return (func) ((gchar *) str1, len1, (gchar *) str2, len2);
-}
-
-gboolean
-tracker_db_interface_sqlite_set_collation_function (TrackerDBInterface       *interface,
-                                                    const gchar              *name,
-                                                    TrackerDBCollationFunc    func)
-{
-	gint result;
-
-	g_return_val_if_fail (TRACKER_IS_DB_INTERFACE (interface), FALSE);
-
-	result = sqlite3_create_collation (interface->db, name, SQLITE_UTF8, func, &collation_function);
-
-	return (result == SQLITE_OK);
-}
-
 gint64
 tracker_db_interface_sqlite_get_last_insert_id (TrackerDBInterface *interface)
 {
@@ -1038,8 +1074,8 @@ tracker_db_statement_class_init (TrackerDBStatementClass *class)
 }
 
 static TrackerDBStatement *
-tracker_db_statement_sqlite_new (TrackerDBInterface       *db_interface,
-                                 sqlite3_stmt             *sqlite_stmt)
+tracker_db_statement_sqlite_new (TrackerDBInterface *db_interface,
+                                 sqlite3_stmt       *sqlite_stmt)
 {
 	TrackerDBStatement *stmt;
 
@@ -1059,6 +1095,8 @@ tracker_db_cursor_finalize (GObject *object)
 
 	cursor = TRACKER_DB_CURSOR (object);
 
+	g_free (cursor->types);
+
 	if (cursor->ref_stmt) {
 		cursor->ref_stmt->stmt_is_sunk = FALSE;
 		tracker_db_statement_sqlite_reset (cursor->ref_stmt);
@@ -1077,10 +1115,11 @@ tracker_db_cursor_iter_next_thread (GSimpleAsyncResult *res,
 {
 	/* run in thread */
 
+	TrackerDBCursor *cursor = TRACKER_DB_CURSOR (object);
 	GError *error = NULL;
 	gboolean result;
 
-	result = tracker_db_cursor_iter_next (TRACKER_DB_CURSOR (object), cancellable, &error);
+	result = db_cursor_iter_next (cursor, cancellable, &error);
 	if (error) {
 		g_simple_async_result_set_from_error (res, error);
 	} else {
@@ -1088,6 +1127,24 @@ tracker_db_cursor_iter_next_thread (GSimpleAsyncResult *res,
 	}
 }
 
+static void
+async_ready_callback_wrapper (GObject      *source_object,
+                              GAsyncResult *res,
+                              gpointer      user_data)
+{
+	TrackerDBCursor *cursor = TRACKER_DB_CURSOR (source_object);
+	TrackerDBInterface *iface = cursor->ref_stmt->db_interface;
+
+	iface->pending = FALSE;
+	if (iface->outstanding_callback) {
+		GAsyncReadyCallback callback = iface->outstanding_callback;
+
+		iface->outstanding_callback = NULL;
+		(callback) (source_object, res, user_data);
+	}
+	g_object_unref (cursor);
+}
+
 static void
 tracker_db_cursor_iter_next_async (TrackerDBCursor     *cursor,
                                    GCancellable        *cancellable,
@@ -1096,7 +1153,11 @@ tracker_db_cursor_iter_next_async (TrackerDBCursor     *cursor,
 {
 	GSimpleAsyncResult *res;
 
-	res = g_simple_async_result_new (G_OBJECT (cursor), callback, user_data, tracker_db_cursor_iter_next_async);
+	g_return_if_fail (!cursor->ref_stmt->db_interface->pending);
+	cursor->ref_stmt->db_interface->pending = TRUE;
+	cursor->ref_stmt->db_interface->outstanding_callback = callback;
+
+	res = g_simple_async_result_new (g_object_ref (cursor), async_ready_callback_wrapper, user_data, tracker_db_cursor_iter_next_async);
 	g_simple_async_result_run_in_thread (res, tracker_db_cursor_iter_next_thread, 0, cancellable);
 }
 
@@ -1119,17 +1180,25 @@ tracker_db_cursor_class_init (TrackerDBCursorClass *class)
 
 	object_class->finalize = tracker_db_cursor_finalize;
 
+	sparql_cursor_class->get_value_type = (TrackerSparqlValueType (*) (TrackerSparqlCursor *, gint)) tracker_db_cursor_get_value_type;
+	sparql_cursor_class->get_variable_name = (const gchar * (*) (TrackerSparqlCursor *, gint)) tracker_db_cursor_get_variable_name;
 	sparql_cursor_class->get_n_columns = (gint (*) (TrackerSparqlCursor *)) tracker_db_cursor_get_n_columns;
 	sparql_cursor_class->get_string = (const gchar * (*) (TrackerSparqlCursor *, gint, glong*)) tracker_db_cursor_get_string;
 	sparql_cursor_class->next = (gboolean (*) (TrackerSparqlCursor *, GCancellable *, GError **)) tracker_db_cursor_iter_next;
 	sparql_cursor_class->next_async = (void (*) (TrackerSparqlCursor *, GCancellable *, GAsyncReadyCallback, gpointer)) tracker_db_cursor_iter_next_async;
 	sparql_cursor_class->next_finish = (gboolean (*) (TrackerSparqlCursor *, GAsyncResult *, GError **)) tracker_db_cursor_iter_next_finish;
 	sparql_cursor_class->rewind = (void (*) (TrackerSparqlCursor *)) tracker_db_cursor_rewind;
+
+	sparql_cursor_class->get_integer = (gint64 (*) (TrackerSparqlCursor *, gint)) tracker_db_cursor_get_int;
+	sparql_cursor_class->get_double = (gdouble (*) (TrackerSparqlCursor *, gint)) tracker_db_cursor_get_double;
+	sparql_cursor_class->get_boolean = (gboolean (*) (TrackerSparqlCursor *, gint)) tracker_db_cursor_get_boolean;
 }
 
 static TrackerDBCursor *
-tracker_db_cursor_sqlite_new (sqlite3_stmt              *sqlite_stmt,
-                              TrackerDBStatement        *ref_stmt)
+tracker_db_cursor_sqlite_new (sqlite3_stmt        *sqlite_stmt,
+                              TrackerDBStatement  *ref_stmt,
+                              TrackerPropertyType *types,
+                              gint                 n_types)
 {
 	TrackerDBCursor *cursor;
 
@@ -1145,13 +1214,23 @@ tracker_db_cursor_sqlite_new (sqlite3_stmt              *sqlite_stmt,
 		cursor->ref_stmt = NULL;
 	}
 
+	if (types) {
+		gint i;
+
+		cursor->types = g_new (TrackerPropertyType, n_types);
+		cursor->n_types = n_types;
+		for (i = 0; i < n_types; i++) {
+			cursor->types[i] = types[i];
+		}
+	}
+
 	return cursor;
 }
 
 void
-tracker_db_statement_bind_double (TrackerDBStatement      *stmt,
-                                  int                      index,
-                                  double                   value)
+tracker_db_statement_bind_double (TrackerDBStatement *stmt,
+                                  int                 index,
+                                  double              value)
 {
 	g_return_if_fail (TRACKER_IS_DB_STATEMENT (stmt));
 
@@ -1161,9 +1240,9 @@ tracker_db_statement_bind_double (TrackerDBStatement      *stmt,
 }
 
 void
-tracker_db_statement_bind_int (TrackerDBStatement       *stmt,
-                               int                       index,
-                               gint64                    value)
+tracker_db_statement_bind_int (TrackerDBStatement *stmt,
+                               int                 index,
+                               gint64              value)
 {
 	g_return_if_fail (TRACKER_IS_DB_STATEMENT (stmt));
 
@@ -1173,8 +1252,8 @@ tracker_db_statement_bind_int (TrackerDBStatement       *stmt,
 }
 
 void
-tracker_db_statement_bind_null (TrackerDBStatement        *stmt,
-                                int                        index)
+tracker_db_statement_bind_null (TrackerDBStatement *stmt,
+                                int                 index)
 {
 	g_return_if_fail (TRACKER_IS_DB_STATEMENT (stmt));
 
@@ -1184,9 +1263,9 @@ tracker_db_statement_bind_null (TrackerDBStatement        *stmt,
 }
 
 void
-tracker_db_statement_bind_text (TrackerDBStatement        *stmt,
-                                int                        index,
-                                const gchar               *value)
+tracker_db_statement_bind_text (TrackerDBStatement *stmt,
+                                int                 index,
+                                const gchar        *value)
 {
 	g_return_if_fail (TRACKER_IS_DB_STATEMENT (stmt));
 
@@ -1199,6 +1278,7 @@ void
 tracker_db_cursor_rewind (TrackerDBCursor *cursor)
 {
 	g_return_if_fail (TRACKER_IS_DB_CURSOR (cursor));
+	g_return_if_fail (!cursor->ref_stmt->db_interface->pending);
 
 	sqlite3_reset (cursor->stmt);
 	cursor->finished = FALSE;
@@ -1208,6 +1288,17 @@ gboolean
 tracker_db_cursor_iter_next (TrackerDBCursor *cursor,
                              GCancellable    *cancellable,
                              GError         **error)
+{
+	g_return_val_if_fail (!cursor->ref_stmt->db_interface->pending, FALSE);
+
+	return db_cursor_iter_next (cursor, cancellable, error);
+}
+
+
+static gboolean
+db_cursor_iter_next (TrackerDBCursor *cursor,
+                     GCancellable    *cancellable,
+                     GError         **error)
 {
 	TrackerDBStatement *stmt = cursor->ref_stmt;
 	TrackerDBInterface *iface = stmt->db_interface;
@@ -1251,7 +1342,9 @@ tracker_db_cursor_get_n_columns (TrackerDBCursor *cursor)
 }
 
 void
-tracker_db_cursor_get_value (TrackerDBCursor *cursor,  guint column, GValue *value)
+tracker_db_cursor_get_value (TrackerDBCursor *cursor,
+                             guint            column,
+                             GValue          *value)
 {
 	gint col_type;
 
@@ -1280,20 +1373,68 @@ tracker_db_cursor_get_value (TrackerDBCursor *cursor,  guint column, GValue *val
 }
 
 gint64
-tracker_db_cursor_get_int (TrackerDBCursor *cursor,  guint column)
+tracker_db_cursor_get_int (TrackerDBCursor *cursor,
+                           guint            column)
 {
 	return (gint64) sqlite3_column_int64 (cursor->stmt, column);
 }
 
 gdouble
-tracker_db_cursor_get_double (TrackerDBCursor *cursor,  guint column)
+tracker_db_cursor_get_double (TrackerDBCursor *cursor,
+                              guint            column)
 {
 	return (gdouble) sqlite3_column_double (cursor->stmt, column);
 }
 
+static gboolean
+tracker_db_cursor_get_boolean (TrackerSparqlCursor *sparql_cursor,
+                               guint                column)
+{
+	TrackerDBCursor *cursor = (TrackerDBCursor *) sparql_cursor;
+	return (g_strcmp0 (sqlite3_column_text (cursor->stmt, column), "true") == 0);
+}
+
+TrackerSparqlValueType
+tracker_db_cursor_get_value_type (TrackerDBCursor *cursor,
+                                  guint            column)
+{
+	gint n_columns = sqlite3_column_count (cursor->stmt);
+
+	g_return_val_if_fail (column < n_columns, TRACKER_SPARQL_VALUE_TYPE_UNBOUND);
+
+	if (sqlite3_column_type (cursor->stmt, column) == SQLITE_NULL) {
+		return TRACKER_SPARQL_VALUE_TYPE_UNBOUND;
+	} else if (column < cursor->n_types) {
+		switch (cursor->types[column]) {
+		case TRACKER_PROPERTY_TYPE_RESOURCE:
+			return TRACKER_SPARQL_VALUE_TYPE_URI;
+		case TRACKER_PROPERTY_TYPE_INTEGER:
+			return TRACKER_SPARQL_VALUE_TYPE_INTEGER;
+		case TRACKER_PROPERTY_TYPE_DOUBLE:
+			return TRACKER_SPARQL_VALUE_TYPE_DOUBLE;
+		case TRACKER_PROPERTY_TYPE_DATETIME:
+			return TRACKER_SPARQL_VALUE_TYPE_DATETIME;
+		case TRACKER_PROPERTY_TYPE_BOOLEAN:
+			return TRACKER_SPARQL_VALUE_TYPE_BOOLEAN;
+		default:
+			return TRACKER_SPARQL_VALUE_TYPE_STRING;
+		}
+	} else {
+		return TRACKER_SPARQL_VALUE_TYPE_STRING;
+	}
+}
 
 const gchar*
-tracker_db_cursor_get_string (TrackerDBCursor *cursor,  guint column, glong *length)
+tracker_db_cursor_get_variable_name (TrackerDBCursor *cursor,
+                                     guint            column)
+{
+	return sqlite3_column_name (cursor->stmt, column);
+}
+
+const gchar*
+tracker_db_cursor_get_string (TrackerDBCursor *cursor,
+                              guint            column,
+                              glong           *length)
 {
 	if (length) {
 		sqlite3_value *val = sqlite3_column_value (cursor->stmt, column);
@@ -1305,10 +1446,9 @@ tracker_db_cursor_get_string (TrackerDBCursor *cursor,  guint column, glong *len
 	}
 }
 
-
 TrackerDBResultSet *
-tracker_db_statement_execute (TrackerDBStatement          *stmt,
-                              GError                     **error)
+tracker_db_statement_execute (TrackerDBStatement  *stmt,
+                              GError             **error)
 {
 	g_return_val_if_fail (!stmt->stmt_is_sunk, NULL);
 
@@ -1316,14 +1456,24 @@ tracker_db_statement_execute (TrackerDBStatement          *stmt,
 }
 
 TrackerDBCursor *
-tracker_db_statement_start_cursor (TrackerDBStatement             *stmt,
-                                   GError                        **error)
+tracker_db_statement_start_cursor (TrackerDBStatement  *stmt,
+                                   GError             **error)
 {
 	g_return_val_if_fail (!stmt->stmt_is_sunk, NULL);
 
-	return tracker_db_cursor_sqlite_new (stmt->stmt, stmt);
+	return tracker_db_cursor_sqlite_new (stmt->stmt, stmt, NULL, 0);
 }
 
+TrackerDBCursor *
+tracker_db_statement_start_sparql_cursor (TrackerDBStatement   *stmt,
+                                          TrackerPropertyType  *types,
+                                          gint                  n_types,
+                                          GError              **error)
+{
+	g_return_val_if_fail (!stmt->stmt_is_sunk, NULL);
+
+	return tracker_db_cursor_sqlite_new (stmt->stmt, stmt, types, n_types);
+}
 
 static void
 tracker_db_statement_init (TrackerDBStatement *stmt)
