@@ -114,9 +114,7 @@ test_backup_and_restore_helper (gboolean journal)
 	/* load data set */
 	data_filename = g_strconcat (data_prefix, ".data", NULL);
 	if (g_file_test (data_filename, G_FILE_TEST_IS_REGULAR)) {
-		tracker_data_begin_db_transaction ();
 		tracker_turtle_reader_load (data_filename, &error);
-		tracker_data_commit_db_transaction ();
 		g_assert_no_error (error);
 	} else {
 		g_assert_not_reached ();
